"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RepoManager = exports.Repo = void 0;
const mongodb_1 = require("mongodb");
const DatabaseManager_1 = require("./DatabaseManager");
function removeReadonlyFields(attributes, ignoreFields) {
    if (!ignoreFields)
        return attributes;
    ignoreFields === null || ignoreFields === void 0 ? void 0 : ignoreFields.forEach((field) => {
        if (attributes[field]) {
            delete attributes[field];
        }
    });
    return attributes;
}
class Repo {
    constructor(collectionName) {
        this.repo = DatabaseManager_1.DatabaseManager.get().collection(collectionName);
    }
    all() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.repo.find({}).toArray();
        });
    }
    find(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.repo.findOne({ _id: new mongodb_1.ObjectId(id) });
        });
    }
    create(model) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.repo.insertOne(model);
            return { ok: Boolean(result.result.ok), id: result.insertedId };
        });
    }
    update(model, ignoreFields) {
        return __awaiter(this, void 0, void 0, function* () {
            model = removeReadonlyFields(model, ignoreFields);
            const result = yield this.repo.updateOne({ _id: new mongodb_1.ObjectId(model._id) }, { $set: model }, { upsert: true });
            return { ok: Boolean(result.result.ok), id: model._id };
        });
    }
    delete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.repo.deleteOne({ _id: new mongodb_1.ObjectId(id) });
            return { ok: Boolean(result.result.ok) };
        });
    }
    search(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.repo.find(query).toArray();
        });
    }
    raw() {
        return this.repo;
    }
}
exports.Repo = Repo;
class RepoManager {
    static get(collectionName) {
        if (!this.repos[collectionName]) {
            this.repos[collectionName] = new Repo(collectionName);
        }
        return this.repos[collectionName];
    }
}
exports.RepoManager = RepoManager;
RepoManager.repos = {};
//# sourceMappingURL=RepoManager.js.map