import { ObjectID, ObjectId } from "mongodb";
import { Operator } from '../definitions/IQueryBuilder';
import { QueryBuilder } from './QueryBuilder';
import { FunctionlessModel } from '../definitions/ModelTypeDefinitions';
export declare type ModelStatic<T extends Model> = {
    new (attributes?: ModelType<T> | {}): T;
};
export declare type ModelType<T extends Model> = FunctionlessModel<T> & {
    _id?: ObjectId;
    createdAt?: string;
    updatedAt?: string;
    cName?: unknown;
    needTimestamp?: unknown;
};
export declare type ModelKey<T extends Model> = keyof FunctionlessModel<T> | '_id';
export declare class Model {
    static collectionName?: string;
    static readonlyFields?: string[];
    static timestamp?: boolean;
    static get modelName(): string;
    get cName(): string;
    static get cName(): string;
    get needTimestamp(): boolean;
    relationships?(): {
        [relationshipName: string]: Function;
    };
    _id?: ObjectId;
    createdAt?: string;
    updatedAt?: string;
    fill(attributes: Partial<ModelType<this>>): void;
    constructor(attributes?: any);
    replicate(): this;
    static first<T extends Model>(this: ModelStatic<T>): Promise<T | undefined>;
    static count<T extends Model>(this: ModelStatic<T>): Promise<number>;
    static all<T extends Model>(this: ModelStatic<T>): Promise<T[]>;
    static find<T extends Model>(this: ModelStatic<T>, primaryKey: string | ObjectID): Promise<T | undefined>;
    static create<T extends Model>(this: ModelStatic<T>, attributes: ModelType<T>): Promise<T>;
    touch(): Promise<void>;
    update(attributes: Partial<ModelType<this>>): Promise<this>;
    save(): Promise<this>;
    delete(): Promise<void>;
    static query<T extends Model>(this: ModelStatic<T>): QueryBuilder<T>;
    static where<T extends Model>(this: ModelStatic<T>, condition: (query: QueryBuilder<T>) => void): QueryBuilder<T>;
    static where<T extends Model>(this: ModelStatic<T>, condition: Partial<ModelType<T>>): QueryBuilder<T>;
    static where<T extends Model>(this: ModelStatic<T>, field: ModelKey<T>, value: any): QueryBuilder<T>;
    static where<T extends Model>(this: ModelStatic<T>, field: ModelKey<T>, operator: Operator, value: any): QueryBuilder<T>;
    static with<T extends Model>(this: ModelStatic<T>, ...relationships: ModelKey<T>[]): QueryBuilder<T>;
    load(...relationships: ModelKey<this>[]): Promise<this>;
    belongsTo<R extends Model>(relationship: new () => R, localKey?: string, foreignKey?: string): QueryBuilder<R>;
    hasOne<R extends Model>(relationship: new () => R, localKey?: string, foreignKey?: string): QueryBuilder<R>;
    hasMany<R extends Model>(relationship: new () => R, localKey?: string, foreignKey?: string): QueryBuilder<R>;
    belongsToMany<R extends Model, P extends Model>(relationship: new () => R, pivot: any, localKey?: string, foreignKey?: string): Promise<QueryBuilder<R>>;
    formatResponse?(cloneSelf: this): any;
    toResponse(): any;
}
