import { QuerySelector } from "mongodb";
import { IQueryBuilder, Operator, PaginationConfig, PaginationResult } from "../definitions/IQueryBuilder";
import { Model, ModelKey, ModelType } from "./Model";
declare type QueryObject<T> = {
    [field: string]: QuerySelector<T>;
};
declare type QueryObject1<T> = QueryObject<T> | {
    $or: (QueryObject<T>[]);
};
declare type QueryObject2<T> = {
    $and: QueryObject1<T>[];
};
export declare type QueryBuilderFunction<T extends Model> = (query: QueryBuilder<T>) => void;
export declare class QueryBuilder<T extends Model> implements IQueryBuilder<T> {
    private repo;
    private modelClass;
    queries: QueryObject2<T>;
    sorter: {
        [field: string]: 1 | -1;
    };
    page?: number;
    perPage?: number;
    relationships?: ModelKey<T>[];
    private lastWhere?;
    private isOne?;
    constructor(collectionName: string, modelClass: Model, relationships?: ModelKey<T>[], isOne?: boolean);
    static query<T extends Model>(collectionName: string, modelClass: Model, relationships?: ModelKey<T>[]): QueryBuilder<T>;
    static where<T extends Model>(field: ModelKey<T>, operator: Operator, value: any, collectionName: string, modelClass: Model): QueryBuilder<T>;
    raw(): import("mongodb").Collection<T>;
    where(condition: (query: QueryBuilder<T>) => void): this;
    where(condition: Partial<ModelType<T>>): this;
    where(field: ModelKey<T>, value: any): this;
    where(field: ModelKey<T>, operator: Operator, value: any): this;
    orWhere(field: ModelKey<T>, value: any): this;
    orWhere(field: ModelKey<T>, operator: Operator, value: any): this;
    whereCondition(condition: QueryBuilderFunction<T> | Partial<ModelType<T>>): this;
    sortBy(field: keyof T, order: 'ASC' | 'DESC'): this;
    paginate(config: PaginationConfig): Promise<PaginationResult<T>>;
    private bindRelationship;
    private cast;
    private fetch;
    get(): Promise<T[]>;
    first(): Promise<T | undefined>;
    count(): Promise<number>;
    delete(): Promise<import("mongodb").DeleteWriteOpResultObject>;
}
export {};
